#!/usr/bin/python
# -*- coding: UTF-8 -*-

# 02.08.2013
#
# Bewegungsmelder mit Ausgang activ low an INT0 des Atmega
# angeschlossen.
# Jeder Flankenwechsel am Alarm-Ausgang löst einen Interrupt am uc aus,
# der eine Nachricht mit dem Pinstatus an den PC sendet.
#
# Der Alarm wird um die Zeitdauer "dauer" verlängtert.
# Wird während der Verzögerungszeit ein neuer Alarm erkannt,
# dann beginnt die Verzögerungszeit neu gesetzt.

import tkinter as Tk
import time, threading
from ser_port import open_ser_port
from global_h import *
from atmega_i import ATMega_i


class Bewegungsmelder(object):
    """ Bewegungsmelder mit Ausschaltverzögerung """

    def __init__(self, master, column = 0, row = 0, irq = PCINT2,
                            width = 50, height = 50, dauer = 5):

        self.irq = irq          # Falls noch andere Interruptquellen vorkommen
        self.dauer = dauer      # die Ausschaltverzögerung nach Alarmende
        self.wait = 0           # decrementiert die Verzögerungszeit
        self.timer = None       # der Timer, der "gecancelt" werden kann

        self.__cv = Tk.Canvas(master, width = width, height = height)
        self.__red = self.__cv.create_arc("4","4", str(int(width) - 1),
                    str(int(width) - 1), style = Tk.CHORD,
                    start = 0, extent = 359, fill = "green", outline = "green")
        self.__cv.grid(column = column, rowspan = 3, row = row, padx = 5, pady = 5)

        Tk.Label(master, text = "Alarmstatus:", width = 15, anchor = "e").grid(
                    column = 1, row = 0, padx = 5, pady = 5, sticky = "nse")
        Tk.Label(master, text = "Wartezeit:",   width = 15, anchor = "e").grid(
                    column = 1, row = 1, padx = 5, pady = 5, sticky = "nse")

        self.lbl1 = Tk.Label(master, text = "inaktiv", width = 8, anchor = "w")
        self.lbl1.grid(column =  2, row = 0, padx = 5, pady = 5, sticky = "nsw")

        self.lbl2 = Tk.Label(master, text = "-", width = 8, anchor = "w")
        self.lbl2.grid(column =  2, row = 1, padx = 5, pady = 5, sticky = "nsw")

    def detect(self, data):
        """ wird bei jedem Flankenwechsel am Ausgang des Bewegungsmelders
            aufgerufen und liefert den aktuellen Status des
            Bewegungsmelder-Ausganges
            Rückgabe: (INT, Sense, Pinstatus)"""
        if data[0] is not self.irq:
            return

        if data[2] > 0:                     # pin auf high - kein Alarm
            self.__cv.itemconfigure(self.__red, fill = "yellow", outline = "yellow")
            self.lbl1.config(text = "warten")

            # den Timer starten, der die Wartezeit prüft
            self.timer = threading.Timer(1, self.check)
            self.timer.start()

            self.wait = self.dauer
            self.lbl2.config(text = str(self.wait))

        else:
            self.__cv.itemconfigure(self.__red, fill = "red", outline = "red")
            self.lbl1.config(text = "aktiv")
            self.lbl1.bell()

            if self.timer is not None:
                self.timer.cancel()
                self.lbl2.config(text = "-")

    def check(self):
        """ wird vom Timer im Sekundenabstand aufgerufen und prüft
            ob die Wartezeit abgelaufen ist. """
        self.wait -= 1
        if self.wait is 0:
            self.__cv.itemconfigure(self.__red, fill = "green", outline = "green")
            self.lbl1.config(text = "inaktiv")
            self.lbl2.config(text = "-")
            self.timer = None

        else:
            self.lbl2.config(text = str(self.wait))
            self.timer = threading.Timer(1, self.check)
            self.timer.start()

if __name__ == "__main__":

    def shutdown(args = None):
        """ Beendet laufende Threads """
        if uc.is_alive():
            uc.stop()

    top = Tk.Tk()
    top.title("Bewegungsmelder")
    top.bind("<Destroy>", shutdown)
    top.resizable(width = False, height = False) # Größe nicht veränderbar

    frm = Tk.LabelFrame(top, text = "Status")
    frm.grid(column = 0, row = 0, padx = 5, pady = 5, sticky = "nsew")

    bw = Bewegungsmelder(frm)

    uc = ATMega_i(open_ser_port())
    uc.add_callback( bw.detect, frame = INTMODE)
    uc.setint(PCINT2, 0x04)   # PinChange an PortD, PD.2

    top.mainloop()