#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# 23.05.2013

# ADXL345 3-Achsen-Beschleunigungssensor
# hier:
# es werden kontinuierliche Messungen ausgeführt
#
# die Daten werden per stream eingelesen, nach 24 Messungen wird ein
# Watermark-Interrupt aufgerufen,
# die ISR liest dann 24 Messwertpaare ein und mittelt die Werte
# Das Hauptprogramm läuft im Sleep().


import time
from ser_port import open_ser_port
from atmega_p import ATMega_p
from global_h import *
from adxl345_h import *

class ADXL345():

    def __init__(self, twi_adr = ADXL_TWI_ADR):
        self.__twi_adr = (twi_adr & 0xFE)
        self.__k = [0,0,0]
        self.__q = 0

    def write_register(self, reg, data):
        """ schreibt ein Byte in ein Register """
        uc.twiwrite(self.__twi_adr, (reg, data))

    def read_register(self, reg):
        """ liest einen Registerinhalt aus
            liefert genau ein Byte zurück"""
        return uc.twiread(self.__twi_adr, regadr = reg)[0]


    def read_data(self):
        """ Liest die Beschleunigungswerte aus
            liefert eine Liste aus 4 Werten zurück """

        t = uc.twiread(self.__twi_adr, regadr = ADXL_DATA, cnt = 6)

        acc = [0,0,0,0]
        self.__q = 0

        for i in range(3):

            #acc[i]  = (t[i*2]) + ((t[(i * 2) +1] & 0x03) << 8)
            #print(int(t[(i*2 + 1)]<<8), int(t[i * 2]))
            acc[i]  = (t[i*2]) + ((t[(i * 2) +1]) << 8)
            #acc[i]  = (t[(i * 2) +1])
            # den Korrekturwert adddieren
            acc[i] += self.__k[i]
            # von unsigned zu integer
            if acc[i] > 2**15:
                acc[i] -= 2**16

            self.__q += acc[i]**2

        self.__q = (self.__q)**0.5
        acc[3] = int(self.__q) - 255

        print("x: {:4} y: {:4} z: {:4} {:4}".format(acc[0], acc[1], acc[2] , acc[3]))
        return acc

    def calibrate(self):
        """ Neue Korrekturwerte für Neutrallage einlesen """

        input("Horizontal ausrichten [return]")
        # Korrekturwerte für die nächste Messung auf Null setzen
        self.__k = [0,0,0]
        # Werte einlesen
        t = self.read_data()

        self.__k[0] = 0 - t[0]
        self.__k[1] = 0 - t[1]
        self.__k[2] = 255  - t[2]


    def init_register(self):
        """ Interrupt wird ausgelöst bei geringer Lageveränderung in beliebiger Richtung """

     	# testen, ob die Chip_ID passt
        print("A")
        x = self.read_register(ADXL_DEVID) # readonly
        print("ID:", x)
        if not x == ADXL_ID:
            print("Err ID")


        #self.write_register(ADXL_TRESH_TAP, (0))
        #self.write_register(ADXL_OFSX, (0))
        #self.write_register(ADXL_OFSY, (0))
        #self.write_register(ADXL_OFSZ, (0))
        #self.write_register(ADXL_DUR, (0))
        #self.write_register(ADXL_LATENT, (0))
        #self.write_register(ADXL_WINDOW, (0))
        #self.write_register(ADXL_TRESH_ACT, (0))
        #self.write_register(ADXL_TRESH_INACT, (0))
        #self.write_register(ADXL_TIME_INACT, (0))
        #self.write_register(ADXL_ACT_INACT_CTL, (0))
        #self.write_register(ADXL_TRESH_FF, (0))
        #self.write_register(ADXL_TIME_FF, (0))
        #self.write_register(ADXL_TAP_AXES, (0))
        #self.write_register(ADXL_ACT_TAP_STATUS, (0)) # read-only
        self.write_register(ADXL_BW_RATE,   BW_RATE_25)
        self.write_register(ADXL_POWER_CTL,  POWER_CTL_MEASURE) # !!!
        self.write_register(ADXL_INT_ENABLE, INT_WATERMARK)
        #self.write_register(ADXL_INT_MAP,  0x00)
        #self.write_register(ADXL_INT_SOURCE,  0x00) # read only
        self.read_register(ADXL_INT_SOURCE) # löscht ggf. gesetzte Bits außer Watermark
        self.write_register(ADXL_DATA_FORMAT, 0| DATA_2G) # DATA_INT_INVERT
        #self.write_register(ADXL_FIFO_CTL,  FIFO_CTL_FIFO_FIFO | ADXL_BLOCK_SIZE)
        self.write_register(ADXL_FIFO_CTL,  FIFO_CTL_FIFO_STREAM | ADXL_BLOCK_SIZE)

        print("Init done")


    def print_registers(self):
        """ Gibt die Werte der relevanten Register aus """
        for i in range(0x1D, 0x3A, 1):
            r = self.read_register(i)
            print("Register: 0x{:02x} -> 0x{:02x} - 0b{:08b}".format(i, r, r))


if __name__ == "__main__":


    def callback(*arg):
        m = [0,0,0]
        for i in range(ADXL_BLOCK_SIZE):
            t = s.read_data()
            m[0] += t[0]
            m[1] += t[1]
            m[2] += t[2]

        m[0] //= ADXL_BLOCK_SIZE
        m[1] //= ADXL_BLOCK_SIZE
        m[2] //= ADXL_BLOCK_SIZE

        for i in m:
            print(hex(i), end = " ")
        print()

    uc = ATMega_p(ser = open_ser_port())

    adxl = ADXL345()
    adxl.init_register()
    adxl.print_registers()
    adxl.calibrate()

    while True:
        time.sleep(1)
        adxl.read_data()






