#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# 12.08.2013

# Bosch BMP085 Temperatur-/Drucksensor - Achtung 3.3 Volt
#

import os, time
from ser_port import open_ser_port
from global_h import *

BMP_TWI_ADR	    = 0xEE
BMP_CONTROL_REG	= 0xF4
BMP_DATA_REG	= 0xF6
BMP_CHIP_ID_REG	= 0xD0
BMP_VERSION_REG	= 0xD1

BMP_READ_TEMPERATURE = 0x2E
BMP_READ_PRESSURE = 0x34


class BMP085(object):

    def __init__(self, altitude = 100, ser = None):
        self.twi_adr = BMP_TWI_ADR
        self.ser = ser
        self.altitude = altitude

        self.ac1 = self.read_int(0xAA)
        self.ac2 = self.read_int(0xAC)
        self.ac3 = self.read_int(0xAE)

        self.ac4 = self.read_uint(0xB0)
        self.ac5 = self.read_uint(0xB2)
        self.ac6 = self.read_uint(0xB4)

        self.b1  = self.read_int(0xB6)
        self.b2  = self.read_int(0xB8)

        self.mb  = self.read_int(0xBA)
        self.mc  = self.read_int(0xBC)
        self.md  = self.read_int(0xBE)

        self.ut = 0
        self.up = 0


    def write(self, adr, byte):
        """ schreibt eine Adresse und ein Byte """
        #out = (STARTBYTE , TWOWIRE, 0x03, self.twi_adr, adr, byte)
        self.ser.write( (STARTBYTE, TWOWIRE, 0x03, self.twi_adr, adr, byte) )


    def read(self, adr, cnt = 1):
        """ schreibt eine Adresse und liest von dort Bytes aus """
        self.ser.write((STARTBYTE, TWOWIRE, 0x02, self.twi_adr, adr))
        self.ser.write((STARTBYTE, TWOWIRE, 0x02, self.twi_adr + 1, cnt))
        self.ser.flushInput()
        i = 10
        while i:
            i -= 1
            a = self.ser.read(1)
            if len(a):
                if a[0] == STARTBYTE:
                    a = self.ser.read(2)
                    if a[0] == TWOWIRE:
                        a = self.ser.read(a[1])
                        break
                    elif a[0] == TWI_ERR:
                        print("TWI-Error")
                        exit()

        if len(a) == 0:
            print("Error")
            exit()
        return a


    def read_int(self, adr):
        """ liest einen 16-Bit signed Integerwert vom Slave aus """

        data = self.read(adr, cnt = 2)
        value = data[1] * 256 + data[2]
        if (value == 2**16) or (value == 0):
            print("Error")
        # falls ein int16 als negativer Wert übertagen wurde,
        # ist das höchstwertige Bit gesetzt
        if value >= 2**15:
            value &= (2**15 - 1)
            value = -(2**15) + value
            #print("negativ")
        return value

    def read_uint(self, adr):
        """ liest einen 16-Bit Unsigned Integerwert vom Slave aus """
        data = self.read(adr, 2)
        value = (data[1] * 256 + data[2])
        if (value == 2**16) or (value == 0):
            print("Error")
        return value

    def convert(self):
        """ Liest die Daten für Temmperatur und Druck ein """
        self.write(BMP_CONTROL_REG, BMP_READ_TEMPERATURE)
        time.sleep(1)
        t  = self.read(BMP_DATA_REG, 2)
        self.ut = t[1] * 256 + t[2]

        self.up = 0         # vier Messungen vornehmen und mitteln
        for i in range(8):
            self.write(BMP_CONTROL_REG, BMP_READ_PRESSURE + (3<<6))
            time.sleep(1)
            t = self.read(BMP_DATA_REG, 2)
            self.up += t[1] * 256 + t[2]
        self.up >>= 3

        #print("cv ready")


    def calc(self):
        """ Berechnet Temperatur und Luftdruck aus den Mess-
            und Kalibrierwerten """

        # der eingelesene Messwert der Temperatur steht bereits in ut,
        # der eingelesene Messwert des Druckes steht bereits in up.

        # --- die Umrechnung der Temperatur gemäß Datenblatt -----------
        x1 = ((self.ut - self.ac6) * self.ac5) >> 15
        x2 = (self.mc << 11) // (x1 + self.md)
        b5 = x1 + x2
        temperature = (b5 + 8) >> 4

        # --- die Umrechnung des Druckes gemäß Datenblatt --------------
        b6 = b5 - 4000
        #print(b6)
        x1 = ((self.b2 * ((b6 * b6) >> 12))) >> 11
        #print(x1)
        x2 = (self.ac2 * b6) >> 11
        #print(x2)
        x3 = x1 + x2
        #print(x3)
        b3 = (( self.ac1 * 4 + x3) + 2)>> 2
        #print(b3)
        x1 = (self.ac3 * b6) >> 13
        #print(x1)
        x2 = (self.b1 * (b6 * b6) >> 12) >> 16
        #print(x2)
        x3 = ((x1 + x2) + 2) >> 2
        #print(x3)
        b4 = (self.ac4 * (x3 + 32768)) >> 15
        #print(b4)
        b7 = ((self.up - b3) * 50000)
        #print(b7)

        if (b7 < 0x80000000):
            p = (b7 << 1) // b4
        else:
            p = (b7 // b4) << 1
        #print(p)

        x1 = (p >> 8) * (p >> 8)
        #print(x1)
        x1 = (x1 * 3038) >> 16
        #print(x1)
        x2 = (-7357 * p) >> 16
        #print(x2)
        pressure = p + ((x1 + x2 + 3791) >> 4) // 10

        pressure /= 100
        print("Temp:", temperature/10)
        print("Druck:", pressure, self.calc_sealevel(pressure))

    def calc_sealevel(self, p):
        return round(p / (1 - (self.altitude/44330))**5.255, 2)


##    def test(self):
##        """ Testwerte gemäß Datenblatt zur Überprüfung des Rechenweges """
##        self.ac1 = 408
##        self.ac2 = -72
##        self.ac3 = -14383
##
##        self.ac4 = 32741
##        self.ac5 = 32757
##        self.ac6 = 23153
##
##        self.b1  = 6190
##        self.b2  = 4
##
##        self.mb  = -32768
##        self.mc  = -8711
##        self.md  = 2868
##
##        self.ut = 27898
##        self.up = 23843


if __name__ == "__main__":


    s = open_ser_port(timeout = 0.05)
    s.flushInput()

    bmp = BMP085(ser = s)
    #s.test()

    #for i in range(2**15 - 10, 2**15 + 10, 1):
    #    print(s.check(i))
    print("10 Sekunden warten ...")
    for i in range(30):
        bmp.convert()
        bmp.calc()
        time.sleep(10)






