#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# 12.08.2013

# Dallas DS1631 Temperatursensor
# voreingestellt im Controllbyte auf 12 Bit (R1 = R0 =1 )!
#
# Das Controllbyte kann ausgelesen werden, das höchstwertige Bit zeigt an,
# ob die Konversion abgeschlossen ist (1 = abgeschlossen). (ohne Funktion ??)
#
# In regelmäßigen Zeiotabständen wird die Temperaturmessung angestoßen.
# Nach der Konversionszeit von 750ms werden 2 Byte ausgelesen
# und in die Temperaturwerte umgerechnet.


import os, time

from global_h import *
from ser_port import open_ser_port
from atmega_p import ATMega_p

DS1631_ADR      = 0x90
START_CONVERT   = 0x51
STOP_CONVERT    = 0x22
READ_TEMPERATUR = 0xAA
ACCESS_CONFIG   = 0xAC
ACCESS_TH       = 0xA1
ACCESS_TL       = 0xA2
POR             = 0x54



class DS1631():
    """ Dallas DS1631 Temperatursensoren """
    def __init__(self, twi_adr = DS1631_ADR):
        self.twi_adr = twi_adr


    def write(self, byte):
        """ schreibt genau ein Byte """
        uc.twiwrite(self.twi_adr, byte)


    def read(self, reg = None, cnt = 1):
        """ liest Bytes vom Slave aus
            schreibt vorher eine Registeradresse (falls angebeben) """
        return uc.twiread(self.twi_adr, regadr = reg, cnt = cnt)


    def calc(self, t):
        """ Berechnet die Temperatur aus dem gelieferten Messwert
            bei gesetztem höchstwertigen Bit liegen negative Temperaturen vor
            je Digit : 0.0625 Grad  """
        if t >= 2048:           # Überlauf ins 12. Bit
            t = t & 2047        # 11 Bit maskieren
            t = -2048 + t
        return t * 0.0625


if __name__ == "__main__":

    def calc_test():
        """ Testen, ob der Übergang von positiven zu negativen
            Temperaturen richtig gerechnet wird """
        for i in range( 25, -1, -1):
            print(s.calc(i))
        for i in range(4095, 4070, -1):
            print(s.calc(i))


    uc = ATMega_p(open_ser_port())

    ds = DS1631(twi_adr = DS1631_ADR)

    a = ds.read(reg = ACCESS_CONFIG)
    print("Config Register: 0x{:02x}".format(a[0]))

    for i in range(15):
        ds.write(START_CONVERT) # Konversion starten

        time.sleep(0.8)         # Konversionszeit abwarten

        a = ds.read(reg = READ_TEMPERATUR, cnt = 2)

        t = ((a[0] * 256 + a[1]) >> 4)
        print(round(ds.calc(t),1))