#!/usr/bin/python
# -*- coding: UTF-8 -*-

# 12.08.2013
#
# Temperaturmessung mit DS18B20
# hier: die Messergebnisse werden via Hintergrundthread empfangen
#
# Die Sensoren werden an den Pins PD2 - PD7 angeschlossen - und benötigen jeweils
# einen Pullup von 4.7k gegen VCC.
#
# Die Sensoren sind jeweils an einem eigenen "Bus" angeschlossen
# (für jeden Sensor wird ein separater Pin benutzt !).
# Per SKIP-ROM + CONVERT erhalten alle Sensoren gleichzeitig den Auftrag zur Temperaturmessung.
# Anschließend werden die Pins/Sensoren nacheinander per READ SCRATCHPAD ausgelesen.
# Diese Methode erfordert weder die Kenntnis noch die Übertragung der ROM CODES der Sensoren.
# Beschränkt die Anzahl der Sensoren auf die Anzahl der freien Pins.

import tkinter as Tk
import time, threading
from ser_port import open_ser_port
from global_h import *
from atmega_i import ATMega_i

class T_anzeige(object):
    """ Ein erläuterndes Label mit anschließender Temperaturanzeige als StringVar() """
    def __init__(self, master, text = "Schalter", column = 0, row = 0):
        self.stringvar = Tk.StringVar()
        self.stringvar.set("nv")

        Tk.Label(master, text = text, width = 13, anchor = "e", font = ('Arial', 10, 'bold')).grid(
                column = column, row = row, padx = 5, pady = 5, sticky = "nsew")
        t = Tk.Label(master, textvariable = self.stringvar, width = 6, anchor = "e", font = ('Arial', 10, 'bold'))
        t.grid(column = column + 1, row = row, padx = 15, pady = 5, sticky = "nsw")

    def update(self, value):
        """ Aktualisiert die Temperaturanzeige """
        self.stringvar.set(value)


class Autosend(threading.Thread):
    """ Stößt in regelmäßigen Abständen eine Temperaturmessung an
        Die Messwerte werden vom Hintergrundthread empfangen
        und an die Anzeige weitergeleitet """

    def __init__(self, sleeptime = 1):
        threading.Thread.__init__(self)
        self.ende = False
        self.sleeptime = sleeptime
        self.start()

    def set_sleeptime(self, sleeptime):
        self.sleeptime = sleeptime

    def stop(self):
        self.ende = True

    def run(self):
        while self.ende is False:
            time.sleep(self.sleeptime)
            uc.ds18b20read(0xFC)

        print("Thread Autosend stopped")


if __name__ == "__main__":

    def shutdown(args = None):
        """ Beendet laufende Threads """
        if a.is_alive():
            a.stop()
        if uc.is_alive():
            uc.stop()

    def update(data):
        """ Aktualisiert die Anzeige """
        for i in range(0, len(data) - 1, 2):
            v = int(data[i] * 256 + data[i+1])
            if v > 2**15:
                v = - 2**16 + v
            if v == -999:
                v = "nv"
            else:
                v = (str(v / 10) + " °C")
            sensorlist[i//2].update(v)


    top = Tk.Tk()
    top.title("Dallas DS18B20")
    top.bind("<Destroy>", shutdown)

    frm = Tk.LabelFrame(top, text = "Temperaturmessung")
    frm.grid(column = 0, row = 0, padx = 5, pady = 5, sticky = "nsew")

    #          Text,              Zeile
    items = (("Sensor 1  (PD2):", 0),
             ("Sensor 2  (PD3):", 1),
             ("Sensor 3  (PD4):", 2),
             ("Sensor 4  (PD5):", 3),
             ("Sensor 5  (PD6):", 4),
             ("Sensor 6  (PD7):", 5) )

    sensorlist = []

    for i in items:
        sensorlist.append(T_anzeige(frm, text = i[0], row = i[1]))

    uc = ATMega_i(open_ser_port())
    uc.add_callback(update, frame = ONEWIRE)

    a = Autosend()

    top.mainloop()