#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# 12.08.2013

# Testet das Lesen vom EEprom mit und ohne Prüfung auf is_reading

# Schreibt Daten auf ein Eeprom und liest sie wieder aus.
# Das Vordergrundprogramm wartet nicht pollend auf die Antwort auf ein Read,
# statt dessen werden eingehende Daten von einem Hintergrundthread empfangen
# und via Callback weitergereicht.

import time
from ser_port import open_ser_port
from global_h import *
from atmega_i import ATMega_i

EEPROM_ADR = 162

class Eeprom():
    """ Klasse zum Lesen und Schreiben auf ein Eeprom
        Die Adressangabe der Daten auf dem Eeprom sowie die Daten
        werden als Tuple/Liste übergeben
        Zu beachten ist die Pagesize beim Write (64/128) Byte :
        Das Write läuft am Ende der Page über und zeigt auf den Pagebeginn. """
    def __init__(self, twi_adr = EEPROM_ADR):
        self.twi_adr = twi_adr


    def write(self, adr = None, data = []):
        """ schreibt eine Liste auf ein Eeprom
            Die Adresse und die Daten werden als Tuple/Liste erwartet
            Pagesize bei der Datenlänge beachten
            Parameter:
            adr  (optional) : Registeradresse (Byte oder Tuple/Liste)
            data (optional) : Daten (Byte oder Tuple/Liste)
            """
        uc.twiwrite(twiadr = self.twi_adr, data = data, regadr = adr)


    def read(self, adr = None, cnt = 1):
        """ liest Bytes vom EEprom aus
            Die Adresse wird als Tuple/Liste erwartet.
            Es können beliebig viele sequentielle Bytes gelesen werden,
            am Ende des Eeprom läuft der Zeiger an den Start des Eeproms über !
            Parameter:
            adr  (optional) : Registeradresse (Byte oder Tuple/Liste)
            data (optional) : Daten (Byte oder Tuple/Liste)
            """
        uc.twiread(twiadr = self.twi_adr, regadr = adr, cnt = cnt)


if __name__ == "__main__":

    def callback(data):
        global rxlist
        for i in range(1, len(data)):
            rxlist.append(i)

    rxlist = []

    ee = Eeprom()

    uc = ATMega_i(open_ser_port())
    uc.add_callback(callback, frame = TWOWIRE)

    data1 = [1,2,3,4,5,6,7,8, 9, 10, 11, 12, 13 ,14, 15 ,16, 17, 18 , 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32]
    data2 = [100, 110, 120, 130, 140, 150, 160, 170, 180, 190, 200, 210, 220,230, 240, 255]

    test = data1        # die Testdaten

    # Startzeit
    starttime = time.time()

    adr = 0
    waits = 0
    bytes = 2048
    while adr < bytes:

        #ee.write(adr = (adr // 256, adr % 256), data = test)
        # und wieder einlesen

        # ohne diese Verzögerung wird der Empfangspuffer überschrieben
        while uc.is_reading:
            waits += 1
            time.sleep(.002)

        ee.read(adr = (adr // 256, adr % 256), cnt = len(test))
        adr += len(test)


    # Endzeit
    endtime = time.time()
    print("geschrieben : {}".format(bytes))
    time.sleep(.5)          # Nachzügler einsammeln

    print("gelesen     : {}".format(len(rxlist)))
    print("waits       : {}".format(waits))
    print("time        : {}".format(endtime - starttime))


    uc.stop()
