#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# 12.08.2013

# Schreibt Daten auf ein Eeprom und liest sie wieder aus.
# Das Programm wartet pollend auf die Antwort auf ein Read.


import time
from ser_port import open_ser_port
from global_h import *
from atmega_p import ATMega_p

EEPROM_ADR = 162  # 0xA2

class Eeprom():
    """ Klasse zum Lesen und Schreiben auf ein Eeprom
        Die Adressangabe der Daten auf dem Eeprom sowie die Daten
        werden als Tuple/Liste übergeben
        Zu beachten ist die Pagesize beim Write (64/128) Byte :
        Das Write läuft am Ende der Page über und zeigt auf den Pagebeginn. """
    def __init__(self, twi_adr = EEPROM_ADR):
        self.twi_adr = twi_adr


    def write(self, adr = None, data = []):
        """ schreibt eine Liste auf ein Eeprom
            Die Adresse und die Daten werden als Tuple/Liste erwartet
            Pagesize bei der Datenlänge beachten
            Parameter:
            adr  (optional) : Registeradresse (Byte oder Tuple/Liste)
            data (optional) : Daten (Byte oder Tuple/Liste)
            """
        uc.twiwrite(twiadr = self.twi_adr, data = data, regadr = adr)


    def read(self, adr = None, cnt = 1):
        """ liest Bytes vom EEprom aus
            Die Adresse wird als Tuple/Liste erwartet.
            Es können beliebig viele sequentielle Bytes gelesen werden,
            am Ende des Eeprom läuft der Zeiger an den Start des Eeproms über !
            """
        return uc.twiread(twiadr = self.twi_adr, regadr = adr, cnt = cnt)


if __name__ == "__main__":

    ee = Eeprom()

    uc = ATMega_p(open_ser_port())

    data1 = [1,2,3,4,5,6,7,8, 9, 10, 11, 12, 13 ,14, 15 ,16]
    data2 = [100, 110, 120, 130, 140, 150, 160, 170, 180, 190, 200, 210, 220, 240, 250, 255 ]

    test = data1

    # Startzeit
    starttime = time.time()

    # Array schreiben
    ee.write(adr = (0,0), data = test)

    # und wieder einlesen
    data = ee.read(adr = (0, 0), cnt = len(test))

    # Endzeit
    endtime = time.time()

    l = []
    for i in data:
        l.append(i)

    print("write : ", test)
    print("read  : ", l)

    print(endtime - starttime)




