#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# 12.08.2013
#
# Der PCF8574 wird beschrieben und ausgelesen.
# Beim Auslesen werden die Daten über einen Hintergrundthread empfangen
# und via Callback durchgereicht

# Es werden 256 Byte geschrieben.
# Nachdem die 256 Byte abgeschickt sind, wird
# - die vergangene Zeit gespeichert
# - die Anzahl der bereits empfangenen Werte (das jeweils zurückgelesene Byte)
#   ausgegeben (es sind weniger als 256 !)
# Nach einer kurzen Wartezeit wird erneut die Anzahl der zurückgelesenen Bytes
# überprüft - nun sind es 256 - wie auch erwartet.

import time
from ser_port import open_ser_port
from global_h import *
from atmega_i import ATMega_i

PCF8574_ADR = 0x40

class Pcf8574():
    """ Klasse zum Lesen und Schreiben auf einen Portexpander PCF8574 """
    def __init__(self, twi_adr = PCF8574_ADR):
        self.twi_adr = twi_adr
        self.list = []

    def write(self, byte):
        """ schreibt ein Byte auf den Portexpander """
        uc.twiwrite(self.twi_adr, data = byte)

    def read(self):
        """ liest ein Byte mit dem Pinstatus vom Portexpander aus """
        #self.waiting = True
        uc.twiread(self.twi_adr)

    def receive(self, data):
        """ nur zum Testen, fügt empfangene Bytes in eine Liste ein
            wird als Callback aufgerufen """
        self.list.append(data)

    def print_list(self):
        """ nur zum Testen, gibt empfangene Byteliste aus """
        #return self.list
        return len(self.list)

if __name__ == "__main__":

    pcf = Pcf8574()

    uc = ATMega_i(ser = open_ser_port())
    uc.add_callback(pcf.receive, frame = TWOWIRE)

    writelist = []
    starttime = time.time()

    for i in range(256):
        pcf.write(i)
        writelist.append(i)
        pcf.read()
        #time.sleep(0.05)

    endtime = time.time()
    print( pcf.print_list())    # Anzahl der bislang empfangenen Bytes
    print(len(writelist))       # Anzahl der bereits gesendeten Bytes

    time.sleep(0.2)             # warten

    print( pcf.print_list())    # Anzahl der empfangenen Bytes nach Wartezeit

    print(endtime - starttime)  # Dauer der Übertragung
    uc.stop()
