#!/usr/bin/python3
# -*- coding: UTF-8 -*-
#
# 12.08.2013
#
# Die I/O-Pins eines PCF8574 sind mit Schaltern bestückt.
# Wenn ein Schalter betätigt wird, dann löst das einen INT aus.
# Der mc erkennt den Interrupt an INT1 und sendet eine Nachricht an den PC
#
# Daraufhin wird der Pinstatus des PCF8574 ausgelesen und es wird
# binär ausgegeben, welche Taste(n) gedrückt sind.
# Durch den Lesevorgang wird der Pin INT am PCF zurückgesetzt.
# Der Interrupt am uC wird in der ISR gelöscht, er muss daher nach jedem Interrupt
# neu gesetzt werden.

import time
from ser_port import open_ser_port
from global_h import *
from atmega_i import ATMega_i

PCF8574_ADR = 0x4E

class PCF8574():
    """ Klasse für den Portexpander PCF8574 """
    def __init__(self, ser = None, twi_adr = PCF8574_ADR):
        self.twi_adr = (twi_adr & 0xFE)
        self.ser = ser
       # self.last_state = 255

    def write(self, byte):
        """ schreibt genau ein Byte  """
        uc.twiwrite(self.twi_adr, byte)

    def read(self):
        """ liest genau ein Byte vom Slave aus """
        uc.twiread(self.twi_adr)

if __name__ == "__main__":

    def irq(data):
        """ Wird aufgerufen, wenn ein Interrupt an INT0 aufgelöst wird.
            Rückgabe: Tuple (Interrupt_Pin_Nr, Pinstatus)
            schaltet den Interrupt auf die nächste erwartete
            Flanke um """

        # Liest den PCF8574 aus um zu erkennen, welche Taste gedrückt ist
        # show_pin_state() holt das Ergebnis ab und zeigt es an
        pcf.read()

        # aktiviert den Interrupt neu - nach einer gewissen Verzögerung durch
        # den Lesevorgang
        set_int1(ANYCHANGE)

    def show_pin_state(data):
        """ Holt das vom PCF8574 ausgelesene Byte ab
            Rückgabe: Tuple (TWI_Adresse, Byte) """
        print("0x{:2x} - {:08b}".format(data[1], 255 - data[1]))


    def set_int1(state):
        """ Setzt den Interrupt INT1 am uController """
        uc.setint(INT1, state)


    def clr_int1():
        """ Löscht den Interrupt INT1 am uController """
        uc.setint(INT1, DELETEINT)


    uc = ATMega_i(open_ser_port())
    uc.add_callback(irq, frame = INTMODE)
    uc.add_callback(show_pin_state, twiadr = PCF8574_ADR)

    pcf = PCF8574()
    pcf.read()              # falls noch ein Interrupt gesetzt ist
                            # dann wird er hier gelöscht

    print("Press key at Pin of PCF8574 ...")

    set_int1(ANYCHANGE)

    while True:
        time.sleep(0.5)
