#!/usr/bin/python
# -*- coding: UTF-8 -*-

# 12.08.2013

# TWI-Kommunikation am Beispiel eines Portexpanders PCF8574
# nach einem TWI-Read wird an der Serielle Schnittstelle pollend
# auf die Antwort gewartet

import tkinter as Tk
from ser_port import open_ser_port
from global_h import *
from atmega_p import ATMega_p
import time
import adr_wahl


class Check_Button(object):
    """ Ein einzelner Check_Button incl. eigener IntVar() """
    def __init__(self, master, width = 10, pin = 0, text = "Schalter",
                 callback = None,  column = 0, row = 0, onvalue = 1):
        self.master = master
        self.cb = callback
        self.checkvar = Tk.IntVar()

        self.chk = Tk.Checkbutton(self.master,
                text = text,
                variable = self.checkvar,
                onvalue = onvalue,
                width = width,
                command = self.cmd)
        self.chk.grid(column = column, row = row, padx = 5, pady = 5, sticky="ew")

    def read(self):
        """ Liefern den Wert der checkvar zurück  """
        return self.checkvar.get()

    def set(self, value):
        """ Setzt einen neuen Wert in der checkvar """
        self.checkvar.set(value)

    def disable(self):
        """ Disabled die CheckButtons eines Slaves,
            dessen Adresse nicht gefunden wurde """
        self.chk.configure(state = Tk.DISABLED)

    def enable(self):
        """ Enabled die CheckButtons eines Slaves,
           dessen Adresse gefunden wurde  """
        self.chk.configure(state = Tk.NORMAL)

    def cmd(self):
        """ Wird ausgeführt, sobald ein Checkbutton geändert wird.
            führt eine Funktion aus der übergeordneten Klasse aus,
            die alle 8 Checkbuttons ausliest, die Werte addiert und
            als Byte an den PCF8574 sendet """
        if not self.cb == None:
            self.cb()


class PCF8574(Tk.LabelFrame):
    """ Stellt 8 Checkbuttons zur Verfügung, mit denen die 8 Ausgangspins
        eines PCF8574 geschaltet werden können """
    def __init__(self, master,
                    width = 10,
                    callback = None,
                    text = "PCF8574",
                    twi_adr = 68,
                    items = None,
                    column = 0,
                    row = 0):
        Tk.LabelFrame.__init__(self,
                    master = master,
                    text = text,
                    width = 200,
                    height = 20)
        self.grid(column = column, row = row, padx = 5, pady = 5, sticky = "nsew")

        self.master = master
        self.callback = callback
        self.twi_adr = twi_adr

        self.items = items
        self.len_items = len(self.items)
        self.check_buttons = []

        for i in range(self.len_items):
            self.check_buttons.append(Check_Button(self, text = self.items[i][0],
                    width = width, callback =self.cmd, onvalue = self.items[i][1],
                    column = self.items[i][2],
                    row = self.items[i][3]))

        aw = adr_wahl.Adr_wahl(top,
                    adr = twi_adr,
                    column = column,
                    row = self.len_items // 2,
                    cb = self.set_adr)
        self.read_state()


    def cmd(self, args = None):
        """ Wird als Callback an Check_Button() weitergereicht und
            wird ausgeführt, sobald ein Checkbutton geändert wird """
        b = 0
        for i in self.check_buttons:
            b += i.read()
        uc.twiwrite(self.twi_adr, b)


    def set_adr(self, adr):
        """ Ändert die aktuelle TWI-Adresse
            und versucht, die aktuellen Pinstates einzulesen. """
        self.twi_adr = adr
        self.read_state()


    def read_state(self):
        """ Liest den PCF8574 aus und setzt die Check_Buttons
            entsprechend der aktuellen Pin-Werte.
            Bei einem TWI-Error werden alle Pins auf 0 gesetzt
            und die Checkbuttons disabled. """
        v = uc.twiread(self.twi_adr)
        for i in range(len(self.check_buttons)):
            if uc.err == False:
                self.check_buttons[i].enable()
                self.check_buttons[i].set((v[0] & (1 << i)))
            else:
                self.check_buttons[i].set(0)
                self.check_buttons[i].disable()


if __name__ == "__main__":


    uc = ATMega_p(open_ser_port())

    top = Tk.Tk()
    top.title("Portexpander PCF8574")

    #          Name, OnValue, Column, Row
    items1 = (("Pin 0",   1, 0, 0),
              ("Pin 1",   2, 0, 1),
              ("Pin 2",   4, 0, 2),
              ("Pin 3",   8, 0, 3),
              ("Pin 4",  16, 1, 0),
              ("Pin 5",  32, 1, 1),
              ("Pin 6",  64, 1, 2),
              ("Pin 7", 128, 1, 3))

    items2 = (("Pin 0",   1, 0, 0),
              ("Pin 1",   2, 0, 1),
              ("Pin 2",   4, 0, 2),
              ("Pin 3",   8, 0, 3),
              ("Pin 4",  16, 1, 0),
              ("Pin 5",  32, 1, 1),
              ("Pin 6",  64, 1, 2),
              ("Pin 7", 128, 1, 3))

    PCF8574(top, text = "PCF8574 (1)", items = items1, column = 0, row = 0, twi_adr = 64)
    PCF8574(top, text = "PCF8574 (2)", items = items2, column = 1, row = 0)

    top.mainloop()
