﻿#! /usr/bin/python3
# -*- coding: UTF-8 -*-

# 12.08.2013

import tkinter as Tk
import pcf8583 as pcf
from ser_port import open_ser_port
from atmega_p import ATMega_p

class Int_Entry(Tk.Entry):
    """ Ein Label plus ein Entry-Widget
        nimmt ausschließlich Integervariablen an """
    def __init__(self, master, label = None, value = 0, column = 0, row = 0, minval = 0, maxval = 59):
        Tk.Label(master, text = label, width = 10, anchor = 'e').grid(column = column, row = row)
        self.__var = Tk.StringVar()
        self.__var.set(str(value))
        self.__minval = minval
        self.__maxval = maxval

        self.__entry  = Tk.Spinbox(
                master,                     # das Parent-Window
                relief = "sunken",
                state = "normal",           #
                from_ = self.__minval,      # Wertebereich von ..
                to = self.__maxval,         # Wertebereich .. bis
                increment = 1,              # Intervall der Werte
                textvariable = self.__var,  # die steuernde Variable
                justify = "left",
                width = 5                   # die Breite des Eingabefeldes
                )

        self.__entry.grid(column = column + 1, row = row,
                          padx = 5, pady = 5, sticky = "nsew")

        self.__check = self.__entry.register(self.isOK)
        self.__entry.configure(validate = 'key',
                               validatecommand = (self.__check, '%P'))

    def set(self, value):
        self.__var.set(value)

    def get(self):
        """ Die Variable kann auch leer sein ! """
        i = self.__var.get()
        if not i:
            i = 0
        return int(i)

    def isOK(self, new_value):
        """ Achtung: self.__var.get() liefert den unveränderten Wert zurück ! """
        if not new_value:            # Eine leeren Eingabe akzeptieren
            self.__entry.bell()
            return True

        if new_value == "-":        # Negative Werte erlauben
            return True

        try:                        # hier die Prüfung, die gelingen muss
            i = int(new_value)
            if i < self.__minval or i > self.__maxval:
                raise BaseException
        except:         # wenn nicht, dann wird die Eingabe rückgängig gemacht
            self.__entry.bell()
            return False

        return True


class App(object):

    def __init__(self):
        self.pcf = pcf.PCF8583(uc = mega)

        self.top = Tk.Tk()
        self.top.title("PCF8583-Zeit einstellen")
        self.top.resizable(width = False, height = False) # Größe nicht veränderbar

        self.frm1 = Tk.LabelFrame(self.top, text = "Datum:",
                        width = 200, height = 100)
        self.frm1.grid(column = 0, row = 0, padx = 5, pady = 5, sticky = "nsew")

        self.dj = Int_Entry(self.frm1, label = "Jahr:", value = 2013,  column = 0, row = 0, minval = 2000, maxval = 2050)
        self.dm = Int_Entry(self.frm1, label = "Monat:", value = 1, column = 0, row = 1, minval = 1, maxval = 12)
        self.dt = Int_Entry(self.frm1, label = "Tag:", value = 1, column = 0, row = 2, minval = 1, maxval = 31)

        self.frm2 = Tk.LabelFrame(self.top, text = "Uhrzeit:",
                        width = 200, height = 100)
        self.frm2.grid(column = 1, row = 0, padx = 5, pady = 5, sticky = "nsew")

        self.th = Int_Entry(self.frm2, label = "Stunde:", value = 0,  column = 0, row = 0, minval = 0, maxval = 23)
        self.tm = Int_Entry(self.frm2, label = "Minute:", value = 0, column = 0, row = 1, minval = 0, maxval = 59)
        self.ts = Int_Entry(self.frm2, label = "Sekunde:", value = 0, column = 0, row = 2, minval = 0, maxval = 59)

        Tk.Button(self.top, text="Zeit auslesen", command=self.read).grid(
                column=0, row=1, padx = 5, pady = 5, sticky="nsew")
        Tk.Button(self.top, text="Zeit schreiben", command=self.write).grid(
                column=0, row=2, padx = 5, pady = 5, sticky="nsew")
        Tk.Button(self.top, text="Ende", command=self.quit).grid(
        column=1, row=2, padx = 5, pady = 5, sticky="nsew")

        self.read()
        self.top.mainloop()


    def write(self):
        """ Schreibt Datum und Zeit auf den PCF8583 """
        self.pcf.settime([self.th.get(), self.tm.get(), self.ts.get()])
        self.pcf.setdate([self.dt.get(), self.dm.get(), self.dj.get()])


    def read(self):
        """ Liest Datum und Zeit vom PCF8583 """
        self.pcf.readtime()
        self.pcf.readdate()
        t = self.pcf.gettime()
        self.dj.set(t[2])
        self.dm.set(t[1])
        self.dt.set(t[0])
        self.th.set(t[3])
        self.tm.set(t[4])
        self.ts.set(t[5])


    def quit(self):
        self.top.destroy()

if __name__ == "__main__":

    mega = ATMega_p(open_ser_port())
    App()

# --- [ eof ] --------------------------------------------------------------
