#!/usr/bin/python
# -*- coding: UTF-8 -*-

# 12.08.2013

import tkinter as Tk
from tkinter import ttk
from ser_port import open_ser_port
from global_h import *
import time


class Check_Button(object):
    """ ein enzelner Checkbutton zusammen mit einer IntVar """
    def __init__(self, master, width = 10, text = "",
                 callback = None,  column = 0, row = 0, pin = None):
        self.master = master
        self.pin = pin
        self.cb = callback
        self.ser = ser
        self.checkvar = Tk.IntVar()

        self.chk = Tk.Checkbutton(self.master, text = text,
                variable = self.checkvar,
                width = width, command = self.cmd)
        self.chk.grid(column = column, row = row, padx = 5, pady = 5, sticky="ew")


    def config(self, state):
        if state:
            self.chk.configure(state = Tk.NORMAL)
        else:
            self.chk.configure(state = Tk.DISABLED)

    def set(self, value):
        self.checkvar.set(value)

    def cmd(self):
        """ Wird ausgeführt, wenn der Checkbutton betätigt wird.
            Gibt die Pin-Nr (in der sowohl der Port als auch der Pin decodiert ist)
            und den Status der Checkbox [0/1] zurück.
        """
        if not self.cb == None:
            self.cb((self.pin, self.checkvar.get()))


class Check_Buttons(Tk.LabelFrame):

    def __init__(self, master,
                    width = 10,
                    callback = None,
                    text = None,
                    items = None,
                    column = 0,
                    row = 0):
        Tk.LabelFrame.__init__(self,
                    master = master,
                    text = text,
                    width = 200,
                    height = 20)
        self.grid(column = column, row = row, padx = 5, pady = 5, sticky = "nsew")

        self.master = master
        self.callback = callback
        self.items = items
        self.len_items = len(self.items)
        self.port = self.items[0][1] // 8           # den Port abgreifen
        self.chk = []
        for i in range(self.len_items):
            self.chk.append(Check_Button(self, text = self.items[i][0],
                        width = width,
                        callback = callback,
                        pin = self.items[i][1],
                        column = self.items[i][2],
                        row = self.items[i][3]))

        ser.write((STARTBYTE, PORTREAD, 0x01, self.port)) ####
        ser.flushInput()
        a = ser.read(1)
        if len(a):
            if a[0] == STARTBYTE:
                a = ser.read(2)
                if a[0] == PORTREAD:
                    a = ser.read(a[1])
                    if len(a)== 4:
                        # PortB/C beginnen mit Pin.0, PortD mit Pin.2
                        if (self.port == 1) or (self.port == 2):
                            j = 0;
                        else:
                            j = 2
                        #print(a, j)

                        for i in self.chk:
                            m = (1<< j)

                            # Checkbutton setzen, wenn als Output gesetzt
                            if a[2] & m:
                                i.set(1)
                            else:
                                i.set(0)
                            j += 1

                            # Outputs enablen, Input disablen
                            if a[1] & m:
                                i.config(1)
                            else:
                                i.config(0)



            else:
                print("error")

if __name__ == "__main__":

    def cb(v):
        """ wird immer ausgeführt, wenn ein Checkbutton geändert wird """
        # print(v)
        port = v[0] // 8
        pinmaske = 1<< v[0] % 8
        if v[1]:
            statemaske = pinmaske
        else:
            statemaske = 0
        ser.write( (STARTBYTE, PORTWRITE, 0x03, port, pinmaske, statemaske) )


    ser = open_ser_port(timeout = 0.1)
    ser.flushInput()

    top = Tk.Tk()
    top.title("Ausgangs-Pins schalten (PORT-Register)")
    top.resizable(width = False, height = False) # Größe nicht veränderbar

    #           Name,     PinNr, Column, Row
    port_b = (("Pin PB.0", 8, 0, 0),
              ("Pin PB.1", 9, 0, 1),
              ("Pin PB.2", 10, 0, 2),
              ("Pin PB.3", 11, 0, 3),
              ("Pin PB.4", 12, 1, 0),
              ("Pin PB.5", 13, 1, 1))

    port_c = (("Pin PC.0", 16, 0, 0),
              ("Pin PC.1", 17, 0, 1),
              ("Pin PC.2", 18, 0, 2),
              ("Pin PC.3", 19, 0, 3))

    port_d = (("Pin PD.2", 26, 0, 2),
              ("Pin PD.3", 27, 0, 3),
              ("Pin PD.4", 28, 1, 0),
              ("Pin PD.5", 29, 1, 1),
              ("Pin PD.6", 30, 1, 2),
              ("Pin PD.7", 31, 1, 3))

    Check_Buttons(top, callback = cb, text = "PORT_B", items = port_b, column = 0, row = 0)
    Check_Buttons(top, callback = cb, text = "PORT_C", items = port_c, column = 1, row = 0)
    Check_Buttons(top, callback = cb, text = "PORT_D", items = port_d, column = 2, row = 0)

    top.mainloop()

