#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# 12.08.2013

# SRF02 Ultrasonic range finder
# via TWI

# hier: die Messwerte werden durch Pollen der Seriellen Schnittstelle empfangen

# Achtung bei den TWI-Adressen:
# das TWI-Modul hier erwartet die Adressen linksbündig  !

import tkinter as Tk
import os, time
import _thread
from atmega_p import ATMega_p

SRF_TWI_ADR	    = 224
SRF_CMD_REG     = 0x00
#SRF_RANGE_REG	= 0x02

#SRF_RANGE_MODE_INCH = 0x50
SRF_RANGE_MODE_CM   = 0x51
#SRF_RANGE_MODE_USEC = 0x52

from ser_port import open_ser_port
from bargraph import BarGraph
from global_h import *
import adr_wahl

class SRF02(object):
    """ SRF02 als Ultraschall-Distanzsensor """
    def __init__(self, twi_adr = SRF_TWI_ADR):
        self.twi_adr = twi_adr
        self.starttime = 0

    def send_ping(self):
        """ sendet das Command zur Messung in der Einheit cm
            merkt sich die Systemzeit, damit nach 70ms
            der Messwert ausgelesen werden kann."""
        self.starttime = time.time()
        uc.twiwrite( self.twi_adr, (SRF_CMD_REG, SRF_RANGE_MODE_CM) )

    def read_ready(self):
        """ prüft, ob die Wartezeit von 70ms abgelaufen ist.
            Wenn ja, dann kann der Messwert ausgelesen werden """
        if (time.time() - self.starttime > 0.065):
            return True
        else:
            return False

    def read_dist(self):
        """ liest die gemessene Distanz (nach Ablauf von 70ms) aus """
        # nach dem Ping (Register 0) steht der interne Zeiger auf Register 1
        # es wird einfach ab Register 1 gelesen, obwohl hier eine Konstante steht
        # aber es dürfen nur die Werte von Register 2 und 3 ausgewertet werden
        return uc.twiread(self.twi_adr, cnt = 3)

    def set_adr(self, adr):
        """  weist eine neue TWI-Adresse zu """
        self.twi_adr = adr

if __name__ == "__main__":

    def autorun():
        """ Thread, der in regelmäßigen Zeitabständen Distanzmessungen ausführt. """
        while thread_run:
            srf.send_ping()
            # die Messdauer von 65ms abwarten
            while not srf.read_ready():
                time.sleep(0.1)

            # die Ausgabe des Messwert veranlassen
            # der Messwert wird im Hintergrund empfangen
            if thread_run:
                data = srf.read_dist()

            if thread_run:
                if not uc.err:
                    bg.draw(data[1]*256 + data[2])
                else:
                    bg.draw(0)

            # die Zeit zwischen den Messungen
            time.sleep(.4)
        print("Thread autorun stopped")

    def shutdown(args = None):
        global thread_run
        thread_run = False

    def callback(adr):
        srf.set_adr(adr)

    top = Tk.Tk()
    top.title("Ultraschall-Entfernungsmessung mit SRF02")
    top.bind("<Destroy>", shutdown)
    top.resizable(width = False, height = False) # Größe nicht veränderbar

    frm = Tk.LabelFrame(top, text = "Distanz in [cm]")
    frm.grid(column = 0, row = 0, padx = 5, pady = 5, columnspan = 1, sticky = "nsew")

    bg  = BarGraph(frm, start_value = 0, end_value = 500, width = 500, heigth = 30, orientation = "e")
    bg.grid(column = 0, row = 0, padx = 5, pady = 5, columnspan = 1, rowspan = 1)

    aw = adr_wahl.Adr_wahl(top, startadr = 224, endadr = 254, adr = 224, cb = callback)
    aw.grid(column = 4, row = 0, padx = 5, pady = 5, sticky = "nsew")

    uc = ATMega_p(ser =  open_ser_port())
    srf = SRF02(twi_adr = 224)

    thread_run = True
    _thread.start_new_thread(autorun,())

    top.mainloop()