#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# 10.07.2013

# Helligkeitssensor TSL2550


import time,  _thread
import tkinter as Tk
from ser_port import open_ser_port
#from twi import transmit
from global_h import *

POWER_DOWN_STATE = 0x00
READ_COMMAND = 0x03
EXTENDED_RANGE_MODE = 0x1D
STANDARD_RANGE_MODE = 0x18
READ_ADC_0 = 0x43
READ_ADC_1 = 0x83


TSL2550_ADR = 0x72

class TSL2550():

    def __init__(self, ser = None, modus = STANDARD_RANGE_MODE):
        self.ser = ser
        self.__modus = modus
        self.__changed = False

    def read(self):
        """ Liest genau 1 Byte aus dem Sensor aus
            schreibt keine Register-Adresse vor dem READ """
        self.ser.write((STARTBYTE, TWOWIRE, 0x02, TSL2550_ADR + 1, 1))
        self,ser.flushInput
        a = ser.read(1)
        if a[0] == STARTBYTE and len(a):
            a = ser.read(2)
            if a[0] == TWOWIRE:
                a = ser.read(a[1])
                return a[1]
        else:
            print("keine Daten empfangen")


    def calc(self, data):
        """ Rechnet den codierten 8-Bit Messwert um """
        chord_nr = (data & 0x70) >> 4
        step_val = 2**(chord_nr)
        step_nr = data & 0x0F
        count = int(16.5 * (2**chord_nr - 1)) + step_nr * 2**chord_nr
#        print("chord_Nr:", chord_nr)
        return count


    def write(self, data):
        """ Sendet ein Byte an die Adresse des Sensors """
        self.ser.write((STARTBYTE, TWOWIRE, 0x02, TSL2550_ADR, data))


    def start_tsl2550(self):
        """ Start die Messung """
        if self.__changed:
            self.__changed = False
            self.write(self.__modus)

        self.write(READ_COMMAND)
        # Wartezeit abhängig vom Modus
        if self.__modus == STANDARD_RANGE_MODE:
            return 0.8
        else:
            return 0.2


    def read_tsl2550(self):
        """ Liest die Messwerte aus """
        self.write(READ_ADC_0)
        adc0 = self.read()

        self.write(READ_ADC_1)
        adc1 = self.read()

        c0 = self.calc(adc0)
        c1 = self.calc(adc1)
        try:
            r = c1 / (c0 - c1)
        except:
            return

        lux = (c0 - c1) * 0.39 * 2.718**((-0.181 * r *r))

        a = round((c1 / c0), 2)
        if a < 0.2:
            t = "[Leuchtstoff]"
        elif a > 0.2:
            t = "[Glühlampe]"
        else:
            t = "[Tageslicht]"

        luxvar.set(str(int(lux)) + " Lux")
        typvar.set(t)
        print(a)
        #return (int(lux), round((c1 / c0), 2))

    def set_mode(self, modus = 0):
        if modus:
            self.__modus = EXTENDED_RANGE_MODE
            print("extend")
        else:
            self.__modus = STANDARD_RANGE_MODE
            print("standard")
        self.changed = True

if __name__ == "__main__":

    def autorun():
        while thread_runs:
            time.sleep(tsl.start_tsl2550())
            if thread_runs:
                tsl.read_tsl2550()
            time.sleep(0.5)
        print("Thread stopped")

    def shutdown(args = None):
        global thread_runs
        thread_runs = False
        if not ser.closed:
           ser.close()

    ser = open_ser_port(timeout = 0.05)
    ser.flushInput()
    tsl = TSL2550(ser = ser)

    top = Tk.Tk()
    top.title("TSL2550")
    top.bind("<Destroy>", shutdown)

    frm = Tk.LabelFrame(top, text = "Beleuchtungsstärke / Lichtfarbe")
    frm.grid(column = 0, row = 0, padx = 5, pady = 5, columnspan = 1, sticky = "nsew")

    luxvar = Tk.IntVar()
    luxvar.set("---")
    Tk.Label(frm, textvar = luxvar, width = 10,  font = ('Arial', 20)).grid(
            column = 0, row = 0, padx = 5, pady = 5, sticky = "nsew")

    typvar = Tk.IntVar()
    typvar.set("---")
    Tk.Label(frm, textvar = typvar, width = 10,  font = ('Arial', 20)).grid(
            column = 1, row = 0, padx = 5, pady = 5, sticky = "nsew")


    thread_runs = True
    _thread.start_new_thread(autorun,())

    top.mainloop()